(
	local debug = off

	local scriptFile = getThisScriptFilename()
	local scriptFolder = getFilenamePath scriptFile

	local infoIni = scriptFolder + "install_info.ini"
	local installTempFolder = "$temp/nikscripts_install/"
		
	local dnProc = dotNetObject "System.Diagnostics.Process"
	local dnCurrentProc = dnProc.GetCurrentProcess()
	local reInitIcons = false
	local maxVer = 1998 + (maxVersion())[1] / 1000

	struct installerInfo
	(
		installDirs = #(),
		installFiles = #(),
		noUpdate = #(),

		scriptName = "Script",
		scriptVer = "",
		scriptMaxVerLow = maxVer,
		scriptMaxVerTop = maxVer,

		fn Initialize =
		(
			local iniName = GetIniSetting infoIni "Info" "Script_name"
			local iniVer = GetIniSetting infoIni "Info" "Script_version"
			local iniMaxLow = (GetIniSetting infoIni "Info" "Maxver_low") as integer
			local iniMaxTop = (GetIniSetting infoIni "Info" "Maxver_top") as integer
			
			if iniName != "" then scriptName = iniName
			if iniVer != "" then scriptVer = iniVer
			if iniMaxLow != undefined then scriptMaxVerLow = iniMaxLow
			if iniMaxTop != undefined then scriptMaxVerTop = iniMaxTop
	
			noUpdate =
			(
				for i in GetIniSetting infoIni "NoUpdate" collect GetIniSetting infoIni "NoUpdate" i
			)
	
		),
		fn Debug =
		(
			clearListener()
			print "Dirs:"
			print installDirs
			print "Files:"
			print installFiles
		),
		init = Initialize()
	)
	installerInfo = installerInfo()
	
	fn SingleMaxProcess =
	(
		local maxProcArr = dnProc.GetProcessesByName("3dsmax")
		local currentMaxFile = dnCurrentProc.MainModule.FileName
		local count = 0
		
		for proc in maxProcArr do
		(
			local file = try (proc.MainModule.FileName) catch ()
			if file == currentMaxFile then count += 1
		)
		count==1
	)
	
	fn DllIsUsed filename =
	(
		local dllused = false

		for i=0 to dnCurrentProc.Modules.count-1 while not dllused do
		(
			local module = dnCurrentProc.Modules.item i
			if pathConfig.pathsResolveEquivalent module.FileName filename then
			(
				dllused = true
			)
		)		
		dllused
	)

	fn IsDll file =
	(
		getFilenameType file == ".dlx"
	)
	
	fn DllIsValidVersion file =
	(
		local fname = getFilenameFile file
		matchPattern fname pattern:("*" + maxVer as string + "*")
	)
	
	fn GetFilesRecursive root pattern =
	(
		local dirArr = #(root)
		local files = #()
		for d in dirArr do
		(
			join dirArr (GetDirectories (d+"/*"))
			files = #()
			for f in dirArr do join files (getFiles (f + pattern))
		)
		files		
	)
	
	fn GetDirsRecursive root =
	(
		local dirArr = GetDirectories (root+"/*")
		for dir in dirArr do
		(
			join dirArr (GetDirectories (dir+"/*"))
		)
		dirArr
	)
	
	fn DelDir dir =
	(
		if DoesFileExist dir do
		(
			dir = symbolicPaths.expandFileName dir
			local delCommand = "RD /S /Q \"" + dir + "\""
			HiddenDOSCommand delCommand startpath:".\\"
		)
	)
	
	fn FilesAreIdentical a b =
	(
		fn formatdate s = substring s 1 (s.count-6)
		
		local filesexist = DoesFileExist a AND DoesFileExist b
		if filesexist then
		(
			local samesize = getFileSize a == getFileSize b
			local samedate = formatdate (getFileModDate a) == formatdate (getFileModDate b)
			samesize AND samedate
		)
		else false
	)
	
	fn GetValidRoots =
	(
		for d in getDirectories (scriptFolder + "*") collect
		(
			local leaf = trimright (pathConfig.stripPathToLeaf d) "\\"
			if not symbolicPaths.isPathName ("$" + leaf) then dontcollect
			else d
		)
	)
	
	fn DeleteUnlockedDll =
	(
		for f in installerInfo.installFiles where f.locked do
		(
			markedFile = f.dest + ".del"
			if doesFileExist markedFile then
			(
				delSuccess = deletefile f.dest
				if delSuccess then
				(
					renamefile markedFile f.dest
					f.locked = false
				)
			)
		)
	)		
	
	fn GenerateInfo =
	(
		local roots = GetValidRoots()
		local filesArr =
		(
			local res = #()
			for r in roots do join res (GetFilesRecursive r "*.*")
			res
		)
		local dirsArr =
		(
			local res = #()
			for r in roots do join res (GetDirsRecursive r)
			res
		)		
		
		struct installStruct (source, dest, root, exist, docopy, locked, autorun)
		
		installerInfo.installFiles = for file in filesArr collect
		(
			local fileStruct = installStruct()
			fileStruct.source = file
			local trimPath = substituteString file scriptfolder ""
			fileStruct.dest = "$" + trimPath
			fileStruct.root = pathConfig.stripPathToTopParent trimPath
			fileStruct.exist = DoesFileExist fileStruct.dest
			fileStruct.autorun =  fileStruct.root == "usermacros" OR fileStruct.root == "userStartupScripts"
			
			fileStruct.docopy =
			(
				if not (FilesAreIdentical file fileStruct.dest) then
				(
					(finditem installerInfo.noUpdate (filenameFromPath file)) == 0
				)
				else false
			)
			
			if IsDll file then
			(
				if DllIsValidVersion file then fileStruct.locked = DllIsUsed fileStruct.dest OR not SingleMaxProcess()
				else continue
			)
			else fileStruct.locked = false
			
			fileStruct
		)
		
		installerInfo.installDirs = for dir in dirsArr collect
		(
			local dirStruct = installStruct()
			dirStruct.source = dir
			dirStruct.dest = substituteString dir scriptfolder "$"
			dirStruct.exist = DoesFileExist dirStruct.dest
			dirStruct
		)
	)
	
	fn ScriptIsInstalled =
	(
		(for f in installerInfo.installFiles where f.exist collect f).count > 0
	)
	
	fn RunExtra type =
	(
		local extraFile = scriptfolder + "_extra_\\" + type + ".ms"
		if DoesFileExist extraFile then
		(
			filein extraFile
		)
	)
	
	fn DeleteEmptyFolders dirpath =
	(
		local files = GetFilesRecursive dirpath "*.*"
		
		if files.count == 0 AND doesFileExist dirpath then
		(
			DelDir dirpath
		)
	)
	
	fn Install =
	(		
		if debug then installerInfo.Debug()
		else
		(
			setwaitcursor()
			RunExtra #install_before
			DeleteUnlockedDll()
			
			local lockedFiles = false 
			
			for d in installerInfo.installDirs where not d.exist do
			(
				makedir d.dest all:true
			)

			for f in installerInfo.installFiles where f.docopy do
			(
				if f.locked then
				(
					renameFile f.dest (f.dest + ".del")
					lockedFiles = true
				)
				else
				(
					deleteFile f.dest
				)
				copyFile f.source f.dest
				if f.root == "usericons" then reInitIcons = true
			)

			for f in installerInfo.installFiles where f.autorun do
			(
				filein f.dest
			)
			
			if lockedFiles then messagebox "Script will be updated at next 3dsMax startup"
			else RunExtra #install_after
			
			setarrowcursor()
		)
	)
	
	fn Uninstall =
	(
		if debug then installerInfo.Debug()
		else
		(
			setwaitcursor()		
			RunExtra #uninstall_before

			local lockedfiles = #()
			for f in installerInfo.installFiles where f.exist do
			(
				if f.locked then appendIfUnique lockedfiles f.dest
				else deleteFile f.dest
			)
			
			for d in installerInfo.installDirs do DeleteEmptyFolders d.dest

			if lockedfiles.count != 0 then
			(
				local opendirs = querybox "Some files are locked.\nShow them in Explorer?" title:"Locked files"
				
				for f in lockedfiles do
				(
					format "File is locked: % \n" f
					if opendirs then ShellLaunch (getFilenamePath f) ""
				)
			)
		
			RunExtra #uninstall_after
			setarrowcursor()
		)
	)
	
	try (destroyDialog ::nsInstallerDialog) catch()
	
	Rollout nsInstallerDialog (installerInfo.scriptName + " " + installerInfo.scriptVer)
	(
		button btnInstall "Install" width:100 height:30 align:#left across:2
		button btnUninstall "Uninstall" width:70 height:30 align:#right enabled:false
		edittext edtLog height:85 labelOnTop:true readOnly:true
		timer tmrReinit interval:100 active:false

		on tmrReinit tick do
		(
			colorman.reInitIcons()
			tmrReinit.active = false
		)

		on btnInstall pressed do
		(
			GenerateInfo()
			Install()
			edtLog.text = "Install completed"
			btnUninstall.enabled = true
			if reInitIcons then tmrReinit.active = true
		)
		on btnUninstall pressed do
		(
			GenerateInfo()
			Uninstall()
			edtLog.text = "Uninstall completed"
			btnUninstall.enabled = false
		)
		on nsInstallerDialog open do
		(
			if maxVer < installerInfo.scriptMaxVerLow OR maxVer > installerInfo.scriptMaxVerTop then
			(
				local reqStr = installerInfo.scriptMaxVerLow as string + " - " + installerInfo.scriptMaxVerTop as string
				edtLog.text = "Script requirements:\n3dsMax " + reqStr
				btnInstall.enabled = btnUninstall.enabled = false
			)
			else
			(
				GenerateInfo()
				btnUninstall.enabled = ScriptIsInstalled()
			)
		)
		
		on nsInstallerDialog close do
		(
			if not debug then DelDir installTempFolder
			nsInstallerDialog = undefined
		)
	)
	createDialog nsInstallerDialog width:200 modal:off style:#(#style_toolwindow, #style_border, #style_sysmenu)
)
